@@ -10,7 +10,7 @@ use lib '../XMLCompile/lib', '../LogReport/lib';
 
 package TestTools;
 use vars '$VERSION';
-$VERSION = '0.65';
+$VERSION = '0.66';
 use base 'Exporter';
 
 use XML::LibXML;
@@ -262,20 +262,14 @@ sub compare_xml($$;$)
 {   my ($tree, $expect, $msg) = @_;
     my $dump = ref $tree ? $tree->toString : $tree;
 
-    if(!defined $dump) { ; }
-    elsif($dump =~ m/\n|\s\s/)
-    {   # output expects superfluous blanks
-        $expect =~ s/\n\z//;
-    }
-    else
-    {   for($expect)
-        {   s/\>\s+/>/gs;
-            s/\s+\</</gs;
-            s/\>\s+\</></gs;
-            s/\s*\n\s*/ /gs;
-            s/\s{2,}/ /gs;
-            s/\s+\z//gs;
-        }
+    for($dump, $expect)
+    {   defined $_ or next;
+        s/\>\s+/>/gs;
+        s/\s+\</</gs;
+        s/\>\s+\</></gs;
+        s/\s*\n\s*/ /gs;
+        s/\s{2,}/ /gs;
+        s/\s+\z//gs;
     }
     is($dump, $expect, $msg);
 }
